import 'package:car_app/service/appwrite/appwrite_authentication.dart';
import 'package:car_app/service/appwrite/appwrite_calendar_event.dart';
import 'package:car_app/service/appwrite/appwrite_route.dart';
import 'package:car_app/service/appwrite/appwrite_team.dart';
import 'package:car_app/service/cached_username_password_authentication.dart';
import 'package:car_app/service/chached_route.dart';
import 'package:car_app/service/interface/authentication.dart';
import 'package:car_app/service/interface/calendar_event.dart';
import 'package:car_app/service/interface/logging.dart';
import 'package:car_app/service/interface/route.dart';
import 'package:car_app/service/interface/team.dart';
import 'package:car_app/service/logging/logger/logger.dart';
import 'package:car_app/service/storage/route_storage.dart';
import 'package:car_app/service/storage/secure_storage.dart';
import 'package:car_app/view/calendar.dart';
import 'package:car_app/view/startup.dart';
import 'package:car_app/view/routes_titles.dart';
import 'package:car_app/viewmodel/authentication.dart';
import 'package:car_app/viewmodel/calendar.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

import 'package:car_app/viewmodel/tracking.dart';
import 'package:path_provider/path_provider.dart';
import 'package:provider/provider.dart';


void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  var applicationDocumentsDirectory = await getApplicationDocumentsDirectory();
  runApp(App(
    applicationDocumentsPath: applicationDocumentsDirectory.path
  ));
}

class App extends StatelessWidget {
  final String applicationDocumentsPath;

  const App({super.key, required this.applicationDocumentsPath});

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: [
        // Providers
        Provider<LoggingService>(create: (context) => Logger(filePath: applicationDocumentsPath, isRelease: kReleaseMode)),
        Provider<AuthenticationService>(create: (context) => CachedUsernamePasswordAuthenticationService(
          authenticationService: AppwriteAuthentication(),
          storageService: SecureStorageService(),
        )),
        Provider<RouteService>(create: (context) => CachedRouteService(
          routeService: AppwriteRoute(),
          storageService: RouteStorageService(
            storageService: SecureStorageService(),
          )
        )),
        Provider<TeamService>(create: (context) => AppwriteTeam(
          authenticationService: Provider.of(context, listen: false),
        )),
        Provider<CalendarEventService>(create: (context) => AppwriteCalendarEventService()),

        // ChangeNotifierProviders
        // currently no better (auto) injection method found, best case would be something similar to https://git.420joos.dev/ost/seproj/studiapp/-/blob/master/StudiApp/StudiApp/MauiProgram.cs
        ChangeNotifierProvider(create: (context) => TrackingViewModel(
          authenticationService: Provider.of(context, listen: false),
          routeService: Provider.of(context, listen: false),
          teamService: Provider.of(context, listen: false),
          loggingService: Provider.of(context, listen: false),
        )),
        ChangeNotifierProvider(create: (context) => AuthenticationViewModel(
          authenticationService: Provider.of(context, listen: false),
        )),
        ChangeNotifierProvider(create: (context) => CalendarViewModel(
          calendarEventService: Provider.of(context, listen: false),
          authenticationService: Provider.of(context, listen: false),
          loggingService: Provider.of(context, listen: false),
        )),
      ],
      child: MaterialApp(
        title: "SharedCar",
        themeMode: ThemeMode.system,
        darkTheme: ThemeData.dark(),
        theme: ThemeData(
          useMaterial3: true,
          colorScheme: ColorScheme.fromSeed(seedColor: Colors.deepPurple)
        ),
        initialRoute: StartupRouting.route,
        routes: {
          StartupRouting.route: (_) => const StartupView(replacementViewRoute: CalendarRouting.route),
          CalendarRouting.route: (_) => const CalendarView(),
        }
      ),
    );
  }
}
