import 'package:car_app/exception/key_not_found.dart';
import 'package:car_app/service/interface/key_value_storage.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';

class SecureStorageService extends KeyValueStorageService<String, String> {
  final _storage = const FlutterSecureStorage();
  
  @override
  Future<void> add(String key, String value) {
    return _storage.write(key: key, value: value);
  }

  @override
  Future<String> get(String key) async {
    var value = await _storage.read(key: key);
    if (value == null) {
      throw KeyNotFoundException(key);
    }

    return value;
  }

  @override
  Future<String> remove(String key) async {
    var value = get(key);
    await _storage.delete(key: key);
    return value;
  }
}
